==> mobroute-0.8.4.tar.gz: mobroute-v0.8.4/api/apirtroute/formatterlegs/formatter_func_legs.go <==
package formatterlegs

import (
	"time"

	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
)

func FormatLegs(
	connectionsVerbose *[]dbt.ConnectionVerbose,
	transfers *[]dbt.Transfer,
	beginTime *time.Time,
	walkspeedKmHr float64,
	from [2]float64,
	to [2]float64,
) *[]RouteLeg {
	var (
		formatYYYYMMDD       = utilfuncs.DayYYYYMMDD(beginTime)
		formatTZUTCDeltasecs = utilfuncs.DayUTCOffsetSecs(beginTime)
		connOrigin           = (*connectionsVerbose)[0]
		connDest             = (*connectionsVerbose)[len(*connectionsVerbose)-1]
		connsGroupedByTrip   = groupConnectionsByTrip(connectionsVerbose)
	)

	// (1): Walk from origin to first stop
	legs := []RouteLeg{*FormatLegWalk(
		from,
		[2]float64{connOrigin.FromStopLat, connOrigin.FromStopLon},
		"Origin",
		connOrigin.FromStopName,
		*beginTime,
		walkspeedKmHr,
	)}

	// (2): Trip & transfers legs between each trip
	for _, tripConnections := range connsGroupedByTrip {
		appendTripLeg := FormatConnectionsToLegTrip(
			formatYYYYMMDD, formatTZUTCDeltasecs, tripConnections,
		)
		if len(legs) > 1 {
			appendTransferLeg := FormatConnectionsToLegTransfer(
				transfers, &legs[len(legs)-1], appendTripLeg,
			)
			legs = append(legs, *appendTransferLeg)
		}
		legs = append(legs, *appendTripLeg)
	}

	// (3): Walk from last stop to destination
	legs = append(legs, *FormatLegWalk(
		[2]float64{connDest.ToStopLat, connDest.ToStopLon},
		to,
		connDest.ToStopName,
		"Destination",
		utilfuncs.CreateTimeFromYYYYMMDDZoneSecs(formatYYYYMMDD, formatTZUTCDeltasecs, connDest.ToStopDRUTCTime),
		walkspeedKmHr,
	))

	return &legs
}

// groupConnectionsByTrip takes an []*dbt.ConnectionVerbose and groups
// returns an array of each []*dbt.ConnectionVerbose containing the connections
// for each trip
func groupConnectionsByTrip(connectionsVerbose *[]dbt.ConnectionVerbose) [][]*dbt.ConnectionVerbose {
	var (
		groupedTrips = [][]*dbt.ConnectionVerbose{}
	)
	for _, c := range *connectionsVerbose {
		if len(groupedTrips) == 0 || groupedTrips[len(groupedTrips)-1][0].TripID != c.TripID {
			groupedTrips = append(groupedTrips, []*dbt.ConnectionVerbose{})
		}
		cc := c
		groupedTrips[len(groupedTrips)-1] = append(groupedTrips[len(groupedTrips)-1], &cc)
	}
	return groupedTrips
}

==> mobroute-0.9.0.tar.gz: mobroute-v0.9.0/api/apirtroute/formatterlegs/formatter_func_legs.go <==
// Package formatterlegs contains the legs formatter used by apirtroute
package formatterlegs

import (
	"time"

	"git.sr.ht/~mil/mobroute/dbt"
	"git.sr.ht/~mil/mobroute/util/utilfuncs"
)

func FormatLegs(
	connectionsVerbose *[]dbt.ConnectionVerbose,
	transfers *[]dbt.Transfer,
	beginTime *time.Time,
	walkspeedKmHr float64,
	from [2]float64,
	to [2]float64,
) *[]RouteLeg {
	var (
		formatYYYYMMDD       = utilfuncs.DayYYYYMMDD(beginTime)
		formatTZUTCDeltasecs = utilfuncs.DayUTCOffsetSecs(beginTime)
		connOrigin           = (*connectionsVerbose)[0]
		connDest             = (*connectionsVerbose)[len(*connectionsVerbose)-1]
		connsGroupedByTrip   = groupConnectionsByTrip(connectionsVerbose)
	)

	// (1): Walk from origin to first stop
	legs := []RouteLeg{*formatLegWalk(
		from,
		[2]float64{connOrigin.FromStopLat, connOrigin.FromStopLon},
		"Origin",
		connOrigin.FromStopName,
		*beginTime,
		walkspeedKmHr,
	)}

	// (2): Trip & transfers legs between each trip
	for _, tripConnections := range connsGroupedByTrip {
		appendTripLeg := formatConnectionsToLegTrip(
			formatYYYYMMDD, formatTZUTCDeltasecs, tripConnections,
		)
		if len(legs) > 1 {
			appendTransferLeg := formatConnectionsToLegTransfer(
				transfers, &legs[len(legs)-1], appendTripLeg,
			)
			legs = append(legs, *appendTransferLeg)
		}
		legs = append(legs, *appendTripLeg)
	}

	// (3): Walk from last stop to destination
	legs = append(legs, *formatLegWalk(
		[2]float64{connDest.ToStopLat, connDest.ToStopLon},
		to,
		connDest.ToStopName,
		"Destination",
		utilfuncs.CreateTimeFromYYYYMMDDZoneSecs(formatYYYYMMDD, formatTZUTCDeltasecs, connDest.ToStopDRUTCTime),
		walkspeedKmHr,
	))

	return &legs
}

// groupConnectionsByTrip takes an []*dbt.ConnectionVerbose and groups
// returns an array of each []*dbt.ConnectionVerbose containing the connections
// for each trip
func groupConnectionsByTrip(connectionsVerbose *[]dbt.ConnectionVerbose) [][]*dbt.ConnectionVerbose {
	var (
		groupedTrips = [][]*dbt.ConnectionVerbose{}
	)
	for _, c := range *connectionsVerbose {
		if len(groupedTrips) == 0 || groupedTrips[len(groupedTrips)-1][0].TripID != c.TripID {
			groupedTrips = append(groupedTrips, []*dbt.ConnectionVerbose{})
		}
		cc := c
		groupedTrips[len(groupedTrips)-1] = append(groupedTrips[len(groupedTrips)-1], &cc)
	}
	return groupedTrips
}
